@@ -76,6 +76,8 @@ class Provider(object):
     :cvar IKOULA: Ikoula driver.
     :cvar OUTSCALE_SAS: Outscale SAS driver.
     :cvar OUTSCALE_INC: Outscale INC driver.
+    :cvar PROFIT_BRICKS: ProfitBricks driver.
+    :cvar VULTR: vultr driver.
     """
     DUMMY = 'dummy'
     EC2 = 'ec2_us_east'
@@ -121,6 +123,9 @@ class Provider(object):
     IKOULA = 'ikoula'
     OUTSCALE_SAS = 'outscale_sas'
     OUTSCALE_INC = 'outscale_inc'
+    VSPHERE = 'vsphere'
+    PROFIT_BRICKS = 'profitbricks'
+    VULTR = 'vultr'
 
     # OpenStack based providers
     HPCLOUD = 'hpcloud'
@@ -184,6 +189,11 @@ class NodeState(object):
     :cvar TERMINATED: Node is terminated. This node can't be started later on.
     :cvar STOPPED: Node is stopped. This node can be started later on.
     :cvar PENDING: Node is pending.
+    :cvar STOPPED: Node is stopped.
+    :cvar SUSPENDED: Node is suspended.
+    :cvar ERROR: Node is an error state. Usually no operations can be performed
+                 on the node once it ends up in the error state.
+    :cvar PAUSED: Node is paused.
     :cvar UNKNOWN: Node state is unknown.
     """
     RUNNING = 0
@@ -192,6 +202,23 @@ class NodeState(object):
     PENDING = 3
     UNKNOWN = 4
     STOPPED = 5
+    SUSPENDED = 6
+    ERROR = 7
+    PAUSED = 8
+
+    @classmethod
+    def tostring(cls, value):
+        values = cls.__dict__
+        values = dict([(key, string) for key, string in values.items() if
+                       not key.startswith('__')])
+
+        for item_key, item_value in values.items():
+            if value == item_value:
+                return item_key
+
+    @classmethod
+    def fromstring(cls, value):
+        return getattr(cls, value.upper(), None)
 
 
 class Architecture(object):
