@@ -23,8 +23,10 @@ from os.path import join as pjoin
 
 try:
     import simplejson as json
+    JSONDecodeError = json.JSONDecodeError
 except ImportError:
     import json
+    JSONDecodeError = ValueError
 
 from libcloud.utils.connection import get_response_object
 
@@ -68,7 +70,7 @@ def get_pricing(driver_type, driver_name, pricing_file_path=None):
     :type driver_type: ``str``
     :param driver_type: Driver type ('compute' or 'storage')
 
-    :type driver_name: ``str`
+    :type driver_name: ``str``
     :param driver_name: Driver name
 
     :type pricing_file_path: ``str``
@@ -95,6 +97,7 @@ def get_pricing(driver_type, driver_name, pricing_file_path=None):
     size_pricing = pricing_data[driver_type][driver_name]
 
     for driver_type in VALID_PRICING_DRIVER_TYPES:
+        # pylint: disable=maybe-no-member
         pricing = pricing_data.get(driver_type, None)
         if pricing:
             PRICING_DATA[driver_type] = pricing
@@ -203,10 +206,11 @@ def download_pricing_file(file_url=DEFAULT_FILE_URL,
     # Verify pricing file is valid
     try:
         data = json.loads(body)
-    except json.decoder.JSONDecodeError:
+    except JSONDecodeError:
         msg = 'Provided URL doesn\'t contain valid pricing data'
         raise Exception(msg)
 
+    # pylint: disable=maybe-no-member
     if not data.get('updated', None):
         msg = 'Provided URL doesn\'t contain valid pricing data'
         raise Exception(msg)
