@@ -25,7 +25,7 @@ try:
 except ImportError:
     import json
 
-from mock import Mock
+from mock import Mock, patch
 
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import method_type
@@ -33,9 +33,6 @@ from libcloud.utils.py3 import u
 
 from libcloud.common.types import InvalidCredsError, MalformedResponseError, \
     LibcloudError
-from libcloud.common.openstack import OpenStackBaseConnection
-from libcloud.common.openstack import OpenStackAuthConnection
-from libcloud.common.openstack import AUTH_TOKEN_EXPIRES_GRACE_SECONDS
 from libcloud.compute.types import Provider, KeyPairDoesNotExistError
 from libcloud.compute.providers import get_driver
 from libcloud.compute.drivers.openstack import (
@@ -85,215 +82,6 @@ class OpenStack_1_0_ResponseTestCase(unittest.TestCase):
             body, RESPONSE_BODY, "Non-XML body should be returned as is")
 
 
-class OpenStackServiceCatalogTests(unittest.TestCase):
-    # TODO refactor and move into libcloud/test/common
-
-    def setUp(self):
-        OpenStackBaseConnection.conn_classes = (OpenStackMockHttp,
-                                                OpenStackMockHttp)
-
-    def test_connection_get_service_catalog(self):
-        connection = OpenStackBaseConnection(*OPENSTACK_PARAMS)
-        connection.auth_url = "https://auth.api.example.com"
-        connection._ex_force_base_url = "https://www.foo.com"
-        connection.driver = OpenStack_1_0_NodeDriver(*OPENSTACK_PARAMS)
-
-        result = connection.get_service_catalog()
-        catalog = result.get_catalog()
-        endpoints = result.get_endpoints('cloudFilesCDN', 'cloudFilesCDN')
-        public_urls = result.get_public_urls('cloudFilesCDN', 'cloudFilesCDN')
-
-        expected_urls = [
-            'https://cdn2.clouddrive.com/v1/MossoCloudFS',
-            'https://cdn2.clouddrive.com/v1/MossoCloudFS'
-        ]
-
-        self.assertTrue('cloudFilesCDN' in catalog)
-        self.assertEqual(len(endpoints), 2)
-        self.assertEqual(public_urls, expected_urls)
-
-
-class OpenStackAuthConnectionTests(unittest.TestCase):
-    # TODO refactor and move into libcloud/test/common
-
-    def setUp(self):
-        OpenStackBaseConnection.auth_url = None
-        OpenStackBaseConnection.conn_classes = (OpenStackMockHttp,
-                                                OpenStackMockHttp)
-
-    def test_auth_url_is_correctly_assembled(self):
-        tuples = [
-            ('1.0', OpenStackMockHttp),
-            ('1.1', OpenStackMockHttp),
-            ('2.0', OpenStack_2_0_MockHttp),
-            ('2.0_apikey', OpenStack_2_0_MockHttp),
-            ('2.0_password', OpenStack_2_0_MockHttp)
-        ]
-
-        APPEND = 0
-        NOTAPPEND = 1
-
-        auth_urls = [
-            ('https://auth.api.example.com', APPEND, ''),
-            ('https://auth.api.example.com/', NOTAPPEND, '/'),
-            ('https://auth.api.example.com/foo/bar', NOTAPPEND, '/foo/bar'),
-            ('https://auth.api.example.com/foo/bar/', NOTAPPEND, '/foo/bar/')
-        ]
-
-        actions = {
-            '1.0': '/v1.0',
-            '1.1': '/v1.1/auth',
-            '2.0': '/v2.0/tokens',
-            '2.0_apikey': '/v2.0/tokens',
-            '2.0_password': '/v2.0/tokens'
-        }
-
-        user_id = OPENSTACK_PARAMS[0]
-        key = OPENSTACK_PARAMS[1]
-
-        for (auth_version, mock_http_class) in tuples:
-            for (url, should_append_default_path, expected_path) in auth_urls:
-                connection = \
-                    self._get_mock_connection(mock_http_class=mock_http_class,
-                                              auth_url=url)
-
-                auth_url = connection.auth_url
-
-                osa = OpenStackAuthConnection(connection,
-                                              auth_url,
-                                              auth_version,
-                                              user_id, key)
-
-                try:
-                    osa = osa.authenticate()
-                except:
-                    pass
-
-                if (should_append_default_path == APPEND):
-                    expected_path = actions[auth_version]
-
-                self.assertEqual(osa.action, expected_path)
-
-    def test_basic_authentication(self):
-        tuples = [
-            ('1.0', OpenStackMockHttp),
-            ('1.1', OpenStackMockHttp),
-            ('2.0', OpenStack_2_0_MockHttp),
-            ('2.0_apikey', OpenStack_2_0_MockHttp),
-            ('2.0_password', OpenStack_2_0_MockHttp)
-        ]
-
-        user_id = OPENSTACK_PARAMS[0]
-        key = OPENSTACK_PARAMS[1]
-
-        for (auth_version, mock_http_class) in tuples:
-            connection = \
-                self._get_mock_connection(mock_http_class=mock_http_class)
-            auth_url = connection.auth_url
-
-            osa = OpenStackAuthConnection(connection, auth_url, auth_version,
-                                          user_id, key)
-
-            self.assertEqual(osa.urls, {})
-            self.assertEqual(osa.auth_token, None)
-            self.assertEqual(osa.auth_user_info, None)
-            osa = osa.authenticate()
-
-            self.assertTrue(len(osa.urls) >= 1)
-            self.assertTrue(osa.auth_token is not None)
-
-            if auth_version in ['1.1', '2.0', '2.0_apikey', '2.0_password']:
-                self.assertTrue(osa.auth_token_expires is not None)
-
-            if auth_version in ['2.0', '2.0_apikey', '2.0_password']:
-                self.assertTrue(osa.auth_user_info is not None)
-
-    def test_token_expiration_and_force_reauthentication(self):
-        user_id = OPENSTACK_PARAMS[0]
-        key = OPENSTACK_PARAMS[1]
-
-        connection = self._get_mock_connection(OpenStack_2_0_MockHttp)
-        auth_url = connection.auth_url
-        auth_version = '2.0'
-
-        yesterday = datetime.datetime.today() - datetime.timedelta(1)
-        tomorrow = datetime.datetime.today() + datetime.timedelta(1)
-
-        osa = OpenStackAuthConnection(connection, auth_url, auth_version,
-                                      user_id, key)
-
-        mocked_auth_method = Mock(wraps=osa.authenticate_2_0_with_body)
-        osa.authenticate_2_0_with_body = mocked_auth_method
-
-        # Force re-auth, expired token
-        osa.auth_token = None
-        osa.auth_token_expires = yesterday
-        count = 5
-
-        for i in range(0, count):
-            osa.authenticate(force=True)
-
-        self.assertEqual(mocked_auth_method.call_count, count)
-
-        # No force reauth, expired token
-        osa.auth_token = None
-        osa.auth_token_expires = yesterday
-
-        mocked_auth_method.call_count = 0
-        self.assertEqual(mocked_auth_method.call_count, 0)
-
-        for i in range(0, count):
-            osa.authenticate(force=False)
-
-        self.assertEqual(mocked_auth_method.call_count, 1)
-
-        # No force reauth, valid / non-expired token
-        osa.auth_token = None
-
-        mocked_auth_method.call_count = 0
-        self.assertEqual(mocked_auth_method.call_count, 0)
-
-        for i in range(0, count):
-            osa.authenticate(force=False)
-
-            if i == 0:
-                osa.auth_token_expires = tomorrow
-
-        self.assertEqual(mocked_auth_method.call_count, 1)
-
-        # No force reauth, valid / non-expired token which is about to expire in
-        # less than AUTH_TOKEN_EXPIRES_GRACE_SECONDS
-        soon = datetime.datetime.utcnow() + \
-            datetime.timedelta(seconds=AUTH_TOKEN_EXPIRES_GRACE_SECONDS - 1)
-        osa.auth_token = None
-
-        mocked_auth_method.call_count = 0
-        self.assertEqual(mocked_auth_method.call_count, 0)
-
-        for i in range(0, count):
-            if i == 0:
-                osa.auth_token_expires = soon
-
-            osa.authenticate(force=False)
-
-        self.assertEqual(mocked_auth_method.call_count, 1)
-
-    def _get_mock_connection(self, mock_http_class, auth_url=None):
-        OpenStackBaseConnection.conn_classes = (mock_http_class,
-                                                mock_http_class)
-
-        if auth_url is None:
-            auth_url = "https://auth.api.example.com"
-
-        OpenStackBaseConnection.auth_url = auth_url
-        connection = OpenStackBaseConnection(*OPENSTACK_PARAMS)
-
-        connection._ex_force_base_url = "https://www.foo.com"
-        connection.driver = OpenStack_1_0_NodeDriver(*OPENSTACK_PARAMS)
-
-        return connection
-
-
 class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
     should_list_locations = False
     should_list_volumes = False
@@ -328,7 +116,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.driver.connection._populate_hosts_and_request_paths()
         clear_pricing_data()
 
-    def test_populate_hosts_and_requests_path(self):
+    @patch('libcloud.common.openstack.OpenStackServiceCatalog')
+    def test_populate_hosts_and_requests_path(self, _):
         tomorrow = datetime.datetime.today() + datetime.timedelta(1)
         cls = self.driver_klass.connectionCls
 
@@ -336,7 +125,7 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
 
         # Test authentication and token re-use
         con = cls('username', 'key')
-        osa = con._osa
+        osa = con.get_auth_class()
 
         mocked_auth_method = Mock()
         osa.authenticate = mocked_auth_method
@@ -357,7 +146,7 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         # ex_force_auth_token provided, authenticate should never be called
         con = cls('username', 'key', ex_force_base_url='http://ponies',
                   ex_force_auth_token='1234')
-        osa = con._osa
+        osa = con.get_auth_class()
 
         mocked_auth_method = Mock()
         osa.authenticate = mocked_auth_method
@@ -827,7 +616,7 @@ class OpenStackMockHttp(MockHttpTestCase):
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
     def _v1_1_auth_INTERNAL_SERVER_ERROR(self, method, url, body, headers):
-        return (httplib.INTERNAL_SERVER_ERROR, "<h1>500: Internal Server Error</h1>",  {'content-type': 'text/html'},
+        return (httplib.INTERNAL_SERVER_ERROR, "<h1>500: Internal Server Error</h1>", {'content-type': 'text/html'},
                 httplib.responses[httplib.INTERNAL_SERVER_ERROR])
 
 
@@ -961,14 +750,24 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         node = nodes[0]
 
         self.assertEqual('12065', node.id)
+
         # test public IPv4
         self.assertTrue('12.16.18.28' in node.public_ips)
         self.assertTrue('50.57.94.35' in node.public_ips)
+
+        # floating ip
+        self.assertTrue('192.168.3.3' in node.public_ips)
+
         # test public IPv6
         self.assertTrue(
             '2001:4801:7808:52:16:3eff:fe47:788a' in node.public_ips)
+
         # test private IPv4
         self.assertTrue('10.182.64.34' in node.private_ips)
+
+        # floating ip
+        self.assertTrue('10.3.3.3' in node.private_ips)
+
         # test private IPv6
         self.assertTrue(
             'fec0:4801:7808:52:16:3eff:fe60:187d' in node.private_ips)
@@ -995,18 +794,34 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual('cd76a3a1-c4ce-40f6-9b9f-07a61508938d', volume.id)
         self.assertEqual('test_volume_2', volume.name)
         self.assertEqual(2, volume.size)
-
-        self.assertEqual(volume.extra['description'], '')
-        self.assertEqual(volume.extra['attachments'][0][
-                         'id'], 'cd76a3a1-c4ce-40f6-9b9f-07a61508938d')
+        self.assertEqual(volume.extra, {
+            'description': '',
+            'attachments': [{
+                'id': 'cd76a3a1-c4ce-40f6-9b9f-07a61508938d',
+                "device": "/dev/vdb",
+                "serverId": "12065",
+                "volumeId": "cd76a3a1-c4ce-40f6-9b9f-07a61508938d",
+            }],
+            'state': 'available',
+            'location': 'nova',
+            'volume_type': 'None',
+            'metadata': {},
+            'created_at': '2013-06-24T11:20:13.000000',
+        })
 
         volume = volumes[1]
         self.assertEqual('cfcec3bc-b736-4db5-9535-4c24112691b5', volume.id)
         self.assertEqual('test_volume', volume.name)
         self.assertEqual(50, volume.size)
-
-        self.assertEqual(volume.extra['description'], 'some description')
-        self.assertEqual(volume.extra['attachments'], [])
+        self.assertEqual(volume.extra, {
+            'description': 'some description',
+            'attachments': [],
+            'state': 'available',
+            'location': 'nova',
+            'volume_type': 'None',
+            'metadata': {},
+            'created_at': '2013-06-21T12:39:02.000000',
+        })
 
     def test_list_sizes(self):
         sizes = self.driver.list_sizes()
@@ -1101,6 +916,18 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(node.name, 'racktest')
         self.assertEqual(node.extra['disk_config'], 'AUTO')
 
+    def test_create_node_with_ex_config_drive(self):
+        OpenStack_1_1_MockHttp.type = 'EX_CONFIG_DRIVE'
+        image = NodeImage(
+            id=11, name='Ubuntu 8.10 (intrepid)', driver=self.driver)
+        size = NodeSize(
+            1, '256 slice', None, None, None, None, driver=self.driver)
+        node = self.driver.create_node(name='racktest', image=image, size=size,
+                                       ex_config_drive=True)
+        self.assertEqual(node.id, '26f7fbee-8ce1-4c28-887a-bfe8e4bb10fe')
+        self.assertEqual(node.name, 'racktest')
+        self.assertTrue(node.extra['config_drive'])
+
     def test_destroy_node(self):
         self.assertTrue(self.node.destroy())
 
@@ -1108,7 +935,9 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertTrue(self.node.reboot())
 
     def test_create_volume(self):
-        self.assertEqual(self.driver.create_volume(1, 'test'), True)
+        volume = self.driver.create_volume(1, 'test')
+        self.assertEqual(volume.name, 'test')
+        self.assertEqual(volume.size, 1)
 
     def test_destroy_volume(self):
         volume = self.driver.ex_get_volume(
@@ -1148,6 +977,16 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
                                          ex_disk_config='MANUAL')
         self.assertTrue(success)
 
+    def test_ex_rebuild_with_ex_config_drive(self):
+        image = NodeImage(id=58, name='Ubuntu 10.10 (intrepid)',
+                          driver=self.driver)
+        node = Node(id=12066, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
+        success = self.driver.ex_rebuild(node, image=image,
+                                         ex_disk_config='MANUAL',
+                                         ex_config_drive=True)
+        self.assertTrue(success)
+
     def test_ex_resize(self):
         size = NodeSize(1, '256 slice', None, None, None, None,
                         driver=self.driver)
